#include <ioavr.h>
#include <inavr.h>

#include "ProjDefs.h"
#include "flash.h"
#include "uart_isr.h"
#include "myio.h"

#define APP_END		0xBFF
#define PAGESIZE	32

INT8U buf[PAGESIZE << 1];

char *myStrings[] =
{
	"\r\nHexBootLoader Ver 0.1\
		\r\nR Execute User Application\
		\r\nU Upload User Program\0",
	"\r\nUploading...\0",
	"\r\nErr Data Format\0",
	"\r\nErr CheckSum\0",
	"\r\nProgramming Ok\0",
	"\r\nErr Buf Overload\0"	
};

#define	MSG_EXEC_USER_APP			0
#define	MSG_UPLOADING_INVITATION	1
#define	MSG_ERR_DATA_FORMAT			2
#define	MSG_ERR_CHECKSUM			3
#define	MSG_PROGRAMMING_OK			4
#define	MSG_ERR_BUF_OVERLOAD		5

//--------------------------------------------------------------

INT8U Char2Hex(char x)
{
	if(('0' <= x) && (x <= '9'))
		return(x - '0');
	else if(('A' <= x) && (x <= 'F'))
		return(x - 'A' + 10);
	else
		return(0);
}

#define WriteMsgId(MsgId) WriteStr(myStrings[MsgId])

void Init(void)
{
	MCFR.all = 0;
	UART_Init();
	putchar = UART_PutChar;
	getchar = UART_GetChar;
	
	GICR = 
		(0 << INT1) | (0 << INT0) |
		(0 << IVSEL) | (1 << IVCE);
	GICR = 
		(0 << INT1) | (0 << INT0) |
		(1 << IVSEL) | (0 << IVCE);
	__enable_interrupt();
}

void	PageWrite(INT16U address)
{
	INT8U Idx;
	INT16U address1 = address;
	INT16U *ptr16u = (INT16U *)buf;;
	
	WriteHex(address >> 8);
	WriteHex(address);
	putchar('W');
	
	for(Idx = 0; Idx < PAGESIZE; Idx++)
	{
		_WAIT_FOR_SPM();
		mySREG = __save_interrupt();
		__disable_interrupt();
			_FILL_TEMP_WORD(address, *(ptr16u++));
			address += 2;
		__restore_interrupt(mySREG);
	}
	_WAIT_FOR_SPM();
	mySREG = __save_interrupt();
	__disable_interrupt();
		_PAGE_WRITE(address1);					// Write Page (Temporary buf will auto erase) R31 = PageNum
	__restore_interrupt(mySREG);
}

void	PageLoad(INT16U address)
{
	INT8U Idx;
	//putchar('\n');
	//putchar('\r');

	WriteHex(address >> 8);
	WriteHex(address);
	putchar('L');
	
	_WAIT_FOR_SPM();
	_ENABLE_RWW_SECTION();
	for(Idx = 0; Idx < (PAGESIZE << 1); Idx++)
		buf[Idx] = _LOAD_PROGRAM_MEMORY(address++);
}

void	PageErase(address)
{
	WriteHex(address >> 8);
	WriteHex(address);
	putchar('E');
	
	_WAIT_FOR_SPM();
	mySREG = __save_interrupt();
	__disable_interrupt();
		_PAGE_ERASE(address);
	__restore_interrupt(mySREG);
}

__C_task void main(void)
{
	char c;
	INT8U Id = 0, subId = 0;
	INT8U RecordNumberOfDataBytes, RecordType, CheckSum;
	INT8U tmp8u, bufPtr, myCounter;
	INT16U RecordStartingAddress, tmp16u;
	void (*AppFuncPtr)( void ) = 0x0000;
	
	/*i = 2000000;
	while(--i);
	while(SPMCR & (1 << SPMEN));
	_PAGE_ERASE(0);
	while(SPMCR & (1 << SPMEN));
	_FILL_TEMP_WORD(0, 0x0102);
	while(SPMCR & (1 << SPMEN));
	_FILL_TEMP_WORD(2, 0x0304);
	while(SPMCR & (1 << SPMEN));
	_PAGE_WRITE(0);
	for(;;);*/
	
	Init();

	for(;;)
	{
			
		switch(Id)
		{
			case	0:
				MCFR.all = 0;
				WriteMsgId(MSG_EXEC_USER_APP);
				Id = 1;
			case	1:
				switch(c = getchar())
				{
					case	'R':
						__disable_interrupt();
						GICR = 
							(0 << INT1) | (0 << INT0) |
							(0 << IVSEL) | (1 << IVCE);
						GICR = 
							(0 << INT1) | (0 << INT0) |
							(0 << IVSEL) | (0 << IVCE);
						_WAIT_FOR_SPM();		
						_ENABLE_RWW_SECTION();
						AppFuncPtr();
						break;
					case	'U':
						WriteMsgId(MSG_UPLOADING_INVITATION);
						bufPtr = 0;
						Id = 2;
						subId = 0;
						break;
				}
				break;
			case	2:
				if(Buf_UART_R_count)
				{
					if((c = getchar()) == ':')
					{
						CheckSum = 0;
						myCounter = 2;
						subId = 1;
					}
					else
					{
						tmp8u <<= 4;
						tmp8u |= Char2Hex(c);
						if(--myCounter == 0)
						{
							myCounter = 2;
		   					switch(subId)
							{
								case	0:
									break;
								case	1:
									RecordNumberOfDataBytes = tmp8u;
									subId = 2;
									break;
								case	2:
									tmp16u = tmp8u;
									subId = 3;
									break;
								case	3:
									tmp16u <<= 8;
									tmp16u |= tmp8u;
									if(!MCFR.IsPrimaryPage)
									{
										MCFR.IsPrimaryPage = 1;
										PageLoad(RecordStartingAddress = tmp16u);
									}
									else
									{
										if((RecordStartingAddress > tmp16u) || ((tmp16u + RecordNumberOfDataBytes) > (RecordStartingAddress + (PAGESIZE << 1))))
										{
											if(MCFR.IsModified)
											{
												PageErase(RecordStartingAddress);
												PageWrite(RecordStartingAddress);
												MCFR.IsModified = 0;
											}
											RecordStartingAddress = tmp16u;
											PageLoad(RecordStartingAddress);
											bufPtr = 0;
										}
									}
									subId = 4;
									break;
								case	4:
									RecordType = tmp8u;
									subId = 5;
									break;
								case	5:
									if(RecordNumberOfDataBytes)
									{
										RecordNumberOfDataBytes--;
										if(bufPtr < (PAGESIZE << 1))
										{
											if(buf[bufPtr] != tmp8u)
											{
												/*putchar('M');
												WriteHex(RecordStartingAddress >> 8);
												WriteHex(RecordStartingAddress);
												WriteHex(bufPtr);
												WriteHex(buf[bufPtr]);
												WriteHex(tmp8u);*/
												buf[bufPtr] = tmp8u;
												MCFR.IsModified = 1;
											}
											bufPtr++;
										}
										else
										{
											WriteMsgId(MSG_ERR_BUF_OVERLOAD);
											Id = 0;
										}
									}
									else
									{
										CheckSum = -CheckSum;
										if(tmp8u == CheckSum)
										{
											if(RecordType == 0)
												CheckSum = tmp8u = 0;
											else if(RecordType == 1)
											{
												if(MCFR.IsModified)
												{
													PageErase(RecordStartingAddress);
													PageWrite(RecordStartingAddress);
												}
												Id = 3;
											}
											else
											{
												WriteMsgId(MSG_ERR_DATA_FORMAT);
												Id = 0;
											}
											
										}
										else
										{
											WriteMsgId(MSG_ERR_CHECKSUM);
											Id = 0;
										}
									}
									break;
							}
							CheckSum += tmp8u;
						}
					}
				}
				break;
			case	3:
				WriteMsgId(MSG_PROGRAMMING_OK);
				Id = 0;
				break;
		}
	}
}
