#include <ioavr.h>
#include <inavr.h>

#include "ProjDefs.h"
#include "mysystypes.h"
#include "uart_isr.h"
#include "myio.h"

#ifndef USR
#define UBRR	UBRRL
#define UCR		UCSRB
#define USR		UCSRA
#endif

void UART_Init(void)
{
	UBRRL = XTAL / (16 * BAUD) - 1;
	UCSRB = ((1 << TXEN) | (1 << RXEN) | (1 << RXCIE));
}

INT8U Buf_UART_R[UART_RX_BUF_SIZE];
volatile INT8U Buf_UART_R_count = 0;

char UART_GetChar(void)
{
	static INT8U Buf_UART_R_tile = 0;
	char my_char; 
	
	while(!Buf_UART_R_count);
	
	mySREG = __save_interrupt();
	__disable_interrupt();
	{
		my_char = Buf_UART_R[Buf_UART_R_tile++];
		#if RX_MASK != 0xFF
		Buf_UART_R_tile &= RX_MASK;
		#endif
		Buf_UART_R_count--;
	}
	__restore_interrupt(mySREG);
	
	return(my_char);
}

#pragma vector = USART_RXC_vect
__interrupt void USART_RXC_ISR(void)
{
	static INT8U Buf_UART_R_head = 0;

	#if RX_MASK == 0xFF
	if(Buf_UART_R_count != 0xFF)
	#else
	if(Buf_UART_R_count < UART_RX_BUF_SIZE)
	#endif
	{
		Buf_UART_R[Buf_UART_R_head++] = UDR;
		Buf_UART_R_count++;
		#if RX_MASK != 0xFF
		Buf_UART_R_head &= RX_MASK;
		#endif
	}
	else
		MCFR.RxBufOverload = 1;
}

INT8U Buf_UART_T[UART_TX_BUF_SIZE];

volatile INT8U Buf_UART_T_count = 0;

#pragma vector = USART_UDRE_vect
__interrupt void USART_UDRE_ISR(void)
{ 
	static INT8U Buf_UART_T_tile = 0;
	
	if(Buf_UART_T_count)
	{
		UDR = Buf_UART_T[Buf_UART_T_tile++];
		#if TX_MASK != 0xFF
		Buf_UART_T_tile &= TX_MASK;
		#endif
		Buf_UART_T_count--;
	}			
	else							 
		ClearBit(UCSRB, UDRIE); 
}			 

char UART_PutChar(char my_char)
{
	static INT8U Buf_UART_T_head = 0;

	#if RX_MASK == 0xFF
	if(Buf_UART_T_count != 0xFF)
	#else
	if(Buf_UART_T_count < UART_TX_BUF_SIZE)
	#endif
	{
		Buf_UART_T[Buf_UART_T_head++] = my_char;
		#if TX_MASK != 0xFF
		Buf_UART_T_head &= TX_MASK;
		#endif
		
		mySREG = __save_interrupt();
		__disable_interrupt();
			Buf_UART_T_count++;
			SetBit(UCSRB, UDRIE);
		__restore_interrupt(mySREG);
	}
	else
		MCFR.TxBufOverload = 1;
	return(my_char);
}							 

void UART_TxFlush(void){while(Buf_UART_T_count);}

